@@ -82,16 +82,25 @@
 
       spec: null,
 
-      visibleResources: [],
       subnets: [],
       members: [],
-      listenerProtocols: ['HTTP', 'TCP', 'TERMINATED_HTTPS'],
+      listenerProtocols: ['HTTP', 'TCP', 'TERMINATED_HTTPS', 'HTTPS'],
+      l7policyActions: ['REJECT', 'REDIRECT_TO_URL', 'REDIRECT_TO_POOL'],
+      l7ruleTypes: ['HOST_NAME', 'PATH', 'FILE_TYPE', 'HEADER', 'COOKIE'],
+      l7ruleCompareTypes: ['REGEX', 'EQUAL_TO', 'STARTS_WITH', 'ENDS_WITH', 'CONTAINS'],
+      l7ruleFileTypeCompareTypes: ['REGEX', 'EQUAL_TO'],
+      poolProtocols: ['HTTP', 'HTTPS', 'PROXY', 'TCP'],
       methods: ['LEAST_CONNECTIONS', 'ROUND_ROBIN', 'SOURCE_IP'],
       types: ['SOURCE_IP', 'HTTP_COOKIE', 'APP_COOKIE'],
-      monitorTypes: ['HTTP', 'PING', 'TCP'],
-      monitorMethods: ['GET', 'HEAD'],
+      monitorTypes: ['HTTP', 'HTTPS', 'PING', 'TCP', 'TLS-HELLO'],
+      monitorMethods: ['GET', 'HEAD', 'POST', 'PUT', 'DELETE',
+                       'TRACE', 'OPTIONS', 'PATCH', 'CONNECT'],
       certificates: [],
       listenerPorts: [],
+      yesNoOptions: [
+        { label: gettext('Yes'), value: true },
+        { label: gettext('No'), value: false }
+      ],
 
       /**
        * api methods for UI controllers
@@ -128,7 +137,6 @@
         submit: null
       };
 
-      model.visibleResources = [];
       model.certificates = [];
       model.listenerPorts = [];
 
@@ -138,43 +146,73 @@
         loadbalancer: {
           name: null,
           description: null,
-          ip: null,
-          subnet: null,
+          vip_address: null,
+          vip_subnet_id: null,
           admin_state_up: true
         },
         listener: {
           id: null,
-          name: gettext('Listener 1'),
+          name: null,
           description: null,
           protocol: null,
-          port: null,
+          protocol_port: null,
           connection_limit: -1,
+          admin_state_up: true,
+          default_pool: null,
+          default_pool_id: null,
+          insert_headers: {},
+          timeout_client_data: 50000,
+          timeout_member_connect: 5000,
+          timeout_member_data: 50000,
+          timeout_tcp_inspect: 0
+        },
+        l7policy: {
+          id: null,
+          name: null,
+          description: null,
+          action: null,
+          position: null,
+          redirect_pool_id: null,
+          redirect_url: null,
+          admin_state_up: true
+        },
+        l7rule: {
+          id: null,
+          type: null,
+          compare_type: null,
+          key: null,
+          rule_value: null,
+          invert: false,
           admin_state_up: true
         },
         pool: {
           id: null,
-          name: gettext('Pool 1'),
+          name: null,
           description: null,
           protocol: null,
-          method: null,
-          type: null,
-          cookie: null,
+          lb_algorithm: null,
+          session_persistence: {
+            type: null,
+            cookie_name: null
+          },
           admin_state_up: true
         },
         monitor: {
           id: null,
+          name: null,
           type: null,
-          interval: 5,
-          retry: 3,
-          retry_down: 3,
+          delay: 5,
+          max_retries: 3,
+          max_retries_down: 3,
           timeout: 5,
-          method: 'GET',
-          status: '200',
-          path: '/',
+          http_method: 'GET',
+          expected_codes: '200',
+          url_path: '/',
           admin_state_up: true
         },
         members: [],
-        certificates: []
+        certificates: [],
+        availablePools: []
       };
 
       if (!model.initializing) {
@@ -195,11 +233,15 @@
       var promise = {
         createloadbalancer: initCreateLoadBalancer,
         createlistener: initCreateListener,
+        createl7policy: initCreateL7Policy,
+        createl7rule: initCreateL7Rule,
         createpool: initCreatePool,
         createmonitor: initCreateMonitor,
         createmembers: initUpdateMemberList,
         editloadbalancer: initEditLoadBalancer,
         editlistener: initEditListener,
+        editl7policy: initEditL7Policy,
+        editl7rule: initEditL7Rule,
         editpool: initEditPool,
         editmonitor: initEditMonitor
       }[type](keymanagerPromise);
@@ -220,7 +262,6 @@
     function initCreateLoadBalancer(keymanagerPromise) {
       model.context.submit = createLoadBalancer;
       return $q.all([
-        lbaasv2API.getLoadBalancers().then(onGetLoadBalancers),
         neutronAPI.getSubnets().then(onGetSubnets),
         neutronAPI.getPorts().then(onGetPorts),
         novaAPI.getServers().then(onGetServers),
@@ -239,16 +280,35 @@
       ]).then(initMemberAddresses);
     }
 
+    function initCreateL7Policy() {
+      model.context.submit = createL7Policy;
+      return lbaasv2API.getListener(model.spec.parentResourceId)
+        .then(onGetListener).then(getPools).then(onGetPools);
+    }
+
+    function initCreateL7Rule() {
+      model.context.submit = createL7Rule;
+      return $q.when();
+    }
+
     function initCreatePool() {
       model.context.submit = createPool;
       //  We get the listener details here because we need to know the listener protocol
       //  in order to default the new pool's protocol to match.
-      return $q.all([
-        lbaasv2API.getListener(model.spec.parentResourceId).then(onGetListener),
-        neutronAPI.getSubnets().then(onGetSubnets),
-        neutronAPI.getPorts().then(onGetPorts),
-        novaAPI.getServers().then(onGetServers)
-      ]).then(initMemberAddresses);
+      if (model.spec.parentResourceId) {
+        return $q.all([
+          lbaasv2API.getListener(model.spec.parentResourceId).then(onGetListener),
+          neutronAPI.getSubnets().then(onGetSubnets),
+          neutronAPI.getPorts().then(onGetPorts),
+          novaAPI.getServers().then(onGetServers)
+        ]).then(initMemberAddresses);
+      } else {
+        return $q.all([
+          neutronAPI.getSubnets().then(onGetSubnets),
+          neutronAPI.getPorts().then(onGetPorts),
+          novaAPI.getServers().then(onGetServers)
+        ]).then(initMemberAddresses);
+      }
     }
 
     function initCreateMonitor() {
@@ -277,12 +337,26 @@
     function initEditListener() {
       model.context.submit = editListener;
       return $q.all([
-        neutronAPI.getSubnets().then(onGetSubnets).then(getListener).then(onGetListener),
+        neutronAPI.getSubnets().then(onGetSubnets).then(getListener)
+          .then(onGetListener).then(getPools).then(onGetPools),
         neutronAPI.getPorts().then(onGetPorts),
         novaAPI.getServers().then(onGetServers)
       ]).then(initMemberAddresses);
     }
 
+    function initEditL7Policy() {
+      model.context.submit = editL7Policy;
+      return lbaasv2API
+        .getListener(model.spec.parentResourceId).then(onGetListener)
+        .then(getPools).then(onGetPools)
+        .then(getL7Policy).then(onGetL7Policy);
+    }
+
+    function initEditL7Rule() {
+      model.context.submit = editL7Rule;
+      return getL7Rule().then(onGetL7Rule);
+    }
+
     function initEditPool() {
       model.context.submit = editPool;
       return $q.all([
@@ -327,6 +401,14 @@
       return lbaasv2API.createListener(spec);
     }
 
+    function createL7Policy(spec) {
+      return lbaasv2API.createL7Policy(spec);
+    }
+
+    function createL7Rule(spec) {
+      return lbaasv2API.createL7Rule(model.spec.parentResourceId, spec);
+    }
+
     function createPool(spec) {
       return lbaasv2API.createPool(spec);
     }
@@ -343,6 +425,14 @@
       return lbaasv2API.editListener(model.context.id, spec);
     }
 
+    function editL7Policy(spec) {
+      return lbaasv2API.editL7Policy(model.context.id, spec);
+    }
+
+    function editL7Rule(spec) {
+      return lbaasv2API.editL7Rule(model.spec.parentResourceId, model.context.id, spec);
+    }
+
     function editPool(spec) {
       return lbaasv2API.editPool(model.context.id, spec);
     }
@@ -358,47 +448,61 @@
     function cleanFinalSpecLoadBalancer(finalSpec) {
       var context = model.context;
 
-      // Load balancer requires subnet
-      if (!finalSpec.loadbalancer.subnet) {
+      // Load balancer requires vip_subnet_id
+      if (!finalSpec.loadbalancer.vip_subnet_id) {
         delete finalSpec.loadbalancer;
       } else {
-        finalSpec.loadbalancer.subnet = finalSpec.loadbalancer.subnet.id;
+        finalSpec.loadbalancer.vip_subnet_id = finalSpec.loadbalancer.vip_subnet_id.id;
       }
 
       // Cannot edit the IP or subnet
       if (context.resource === 'loadbalancer' && context.id) {
-        delete finalSpec.subnet;
-        delete finalSpec.ip;
+        delete finalSpec.vip_subnet_id;
+        delete finalSpec.vip_address;
       }
     }
 
     function cleanFinalSpecListener(finalSpec) {
-      if (!finalSpec.listener.protocol || !finalSpec.listener.port) {
+      if (!finalSpec.listener.protocol || !finalSpec.listener.protocol_port) {
         // Listener requires protocol and port
         delete finalSpec.listener;
         delete finalSpec.certificates;
-      } else if (finalSpec.listener.protocol !== 'TERMINATED_HTTPS') {
-        // Remove certificate containers if not using TERMINATED_HTTPS
-        delete finalSpec.certificates;
       } else {
-        var containers = [];
-        angular.forEach(finalSpec.certificates, function(cert) {
-          containers.push(cert.id);
-        });
-        finalSpec.certificates = containers;
+        for (var header in finalSpec.listener.insert_headers) {
+          if (!finalSpec.listener.insert_headers[header]) {
+            delete finalSpec.listener.insert_headers[header];
+          }
+        }
+        if (finalSpec.listener.protocol !== 'TERMINATED_HTTPS') {
+          // Remove certificate containers if not using TERMINATED_HTTPS
+          delete finalSpec.certificates;
+        } else {
+          var containers = [];
+          angular.forEach(finalSpec.certificates, function(cert) {
+            containers.push(cert.id);
+          });
+          finalSpec.certificates = containers;
+        }
       }
     }
 
     function cleanFinalSpecPool(finalSpec) {
 
       // Pool requires method
-      if (!finalSpec.pool.method) {
+      if (!finalSpec.pool.lb_algorithm) {
         delete finalSpec.pool;
       } else {
         // The pool protocol must be HTTP if the listener protocol is TERMINATED_HTTPS and
         // otherwise has to match it.
         var protocol = finalSpec.listener ? finalSpec.listener.protocol : finalSpec.pool.protocol;
         finalSpec.pool.protocol = protocol === 'TERMINATED_HTTPS' ? 'HTTP' : protocol;
+        if (angular.isObject(finalSpec.pool.session_persistence)) {
+          if (!finalSpec.pool.session_persistence.type) {
+            finalSpec.pool.session_persistence = null;
+          } else if (finalSpec.pool.session_persistence.type !== 'APP_COOKIE') {
+            finalSpec.pool.session_persistence.cookie_name = null;
+          }
+        }
       }
     }
 
@@ -410,20 +514,20 @@
 
       var members = [];
       angular.forEach(finalSpec.members, function cleanMember(member) {
-        if (member.address && member.port) {
-          var propsToRemove = ['name', 'description', 'addresses', 'allocatedMember'];
+        if (member.address && member.protocol_port) {
+          var propsToRemove = ['description', 'addresses', 'allocatedMember'];
           propsToRemove.forEach(function deleteProperty(prop) {
             if (angular.isDefined(member[prop])) {
               delete member[prop];
             }
           });
           if (angular.isObject(member.address)) {
-            member.subnet = member.address.subnet;
+            member.subnet_id = member.address.subnet;
             member.address = member.address.ip;
-          } else if (member.subnet) {
-            member.subnet = member.subnet.id;
+          } else if (member.subnet_id) {
+            member.subnet_id = member.subnet_id.id;
           } else {
-            delete member.subnet;
+            delete member.subnet_id;
           }
           members.push(member);
         }
@@ -437,9 +541,9 @@
 
     function cleanFinalSpecMonitor(finalSpec) {
 
-      // Monitor requires an interval, retry count, and timeout
-      if (!angular.isNumber(finalSpec.monitor.interval) ||
-          !angular.isNumber(finalSpec.monitor.retry) ||
+      // Monitor requires delay, max_retries, and timeout
+      if (!angular.isNumber(finalSpec.monitor.delay) ||
+          !angular.isNumber(finalSpec.monitor.max_retries) ||
           !angular.isNumber(finalSpec.monitor.timeout)) {
         delete finalSpec.monitor;
         return;
@@ -447,9 +551,9 @@
 
       // Only include necessary monitor properties
       if (finalSpec.monitor.type !== 'HTTP') {
-        delete finalSpec.monitor.method;
-        delete finalSpec.monitor.status;
-        delete finalSpec.monitor.path;
+        delete finalSpec.monitor.http_method;
+        delete finalSpec.monitor.expected_codes;
+        delete finalSpec.monitor.url_path;
       }
     }
 
@@ -463,33 +567,25 @@
       });
     }
 
-    function onGetLoadBalancers(response) {
-      var existingNames = {};
-      angular.forEach(response.data.items, function nameExists(lb) {
-        existingNames[lb.name] = 1;
-      });
-      var name;
-      var index = response.data.items.length;
-      do {
-        index += 1;
-        name = interpolate(gettext('Load Balancer %(index)s'), { index: index }, true);
-      } while (name in existingNames);
-      model.spec.loadbalancer.name = name;
-    }
-
     function onGetListeners(response) {
-      var existingNames = {};
-      angular.forEach(response.data.items, function nameExists(listener) {
-        existingNames[listener.name] = 1;
+      angular.forEach(response.data.items, function addPort(listener) {
         model.listenerPorts.push(listener.protocol_port);
       });
-      var name;
-      var index = response.data.items.length;
-      do {
-        index += 1;
-        name = interpolate(gettext('Listener %(index)s'), { index: index }, true);
-      } while (name in existingNames);
-      model.spec.listener.name = name;
+    }
+
+    function onGetPools(response) {
+      angular.forEach(response.data.items, function addPool(pool) {
+        if (pool.listeners.length > 0 && pool.listeners[0].id !== model.spec.listener.id) {
+          return;
+        }
+        var p = pool.id + '(' + pool.name + ')';
+        if (pool.protocol === model.spec.listener.protocol) {
+          model.spec.availablePools.push(p);
+        } else if (pool.protocol === 'HTTP' &&
+          model.spec.listener.protocol === 'TERMINATED_HTTPS') {
+          model.spec.availablePools.push(p);
+        }
+      });
     }
 
     function onGetSubnets(response) {
@@ -509,7 +605,8 @@
             weight: 1,
             monitor_address: null,
             monitor_port: null,
-            admin_state_up: true
+            admin_state_up: true,
+            backup: false
           });
         }
       });
@@ -541,7 +638,7 @@
         member.address = member.addresses[0];
 
         if (model.spec.pool.protocol) {
-          member.port = {HTTP: 80}[model.spec.pool.protocol];
+          member.protocol_port = {HTTP: 80}[model.spec.pool.protocol];
         }
       });
     }
@@ -550,10 +647,23 @@
       return lbaasv2API.getListener(model.context.id, true);
     }
 
+    function getL7Policy() {
+      return lbaasv2API.getL7Policy(model.context.id, true);
+    }
+
+    function getL7Rule() {
+      return lbaasv2API.getL7Rule(model.spec.parentResourceId,
+        model.context.id);
+    }
+
     function getPool() {
       return lbaasv2API.getPool(model.context.id, true);
     }
 
+    function getPools() {
+      return lbaasv2API.getPools(model.spec.loadbalancer_id);
+    }
+
     function getMembers() {
       return lbaasv2API.getMembers(model.spec.parentResourceId);
     }
@@ -569,7 +679,6 @@
       setListenerSpec(result.listener || result);
 
       if (result.listener) {
-        model.visibleResources.push('listener');
         model.spec.loadbalancer_id = result.listener.load_balancers[0].id;
 
         if (result.listener.protocol === 'TERMINATED_HTTPS') {
@@ -582,15 +691,12 @@
               });
             });
           });
-          model.visibleResources.push('certificates');
           $('#wizard-side-nav ul li:last').show();
         }
       }
 
       if (result.pool) {
         setPoolSpec(result.pool);
-        model.visibleResources.push('pool');
-        model.visibleResources.push('members');
 
         if (result.members) {
           setMembersSpec(result.members);
@@ -598,27 +704,34 @@
 
         if (result.monitor) {
           setMonitorSpec(result.monitor);
-          model.visibleResources.push('monitor');
         }
       }
     }
 
+    function onGetL7Policy(response) {
+      var result = response.data;
+
+      setL7PolicySpec(result.l7policy || result);
+    }
+
+    function onGetL7Rule(response) {
+      var result = response.data;
+
+      setL7RuleSpec(result.l7rule || result);
+    }
+
     function onGetPool(response) {
       var result = response.data;
 
       setPoolSpec(result.pool || result);
 
       if (result.pool) {
-        model.visibleResources.push('pool');
-        model.visibleResources.push('members');
-
         if (result.members) {
           setMembersSpec(result.members);
         }
 
         if (result.monitor) {
           setMonitorSpec(result.monitor);
-          model.visibleResources.push('monitor');
         }
       }
     }
@@ -627,8 +740,8 @@
       var spec = model.spec.loadbalancer;
       spec.name = loadbalancer.name;
       spec.description = loadbalancer.description;
-      spec.ip = loadbalancer.vip_address;
-      spec.subnet = loadbalancer.vip_subnet_id;
+      spec.vip_address = loadbalancer.vip_address;
+      spec.vip_subnet_id = loadbalancer.vip_subnet_id;
       spec.admin_state_up = loadbalancer.admin_state_up;
     }
 
@@ -638,9 +751,38 @@
       spec.name = listener.name;
       spec.description = listener.description;
       spec.protocol = listener.protocol;
-      spec.port = listener.protocol_port;
+      spec.protocol_port = listener.protocol_port;
       spec.connection_limit = listener.connection_limit;
       spec.admin_state_up = listener.admin_state_up;
+      spec.default_pool_id = listener.default_pool_id;
+      spec.insert_headers = listener.insert_headers;
+      spec.timeout_client_data = listener.timeout_client_data;
+      spec.timeout_member_connect = listener.timeout_member_connect;
+      spec.timeout_member_data = listener.timeout_member_data;
+      spec.timeout_tcp_inspect = listener.timeout_tcp_inspect;
+    }
+
+    function setL7PolicySpec(l7policy) {
+      var spec = model.spec.l7policy;
+      spec.id = l7policy.id;
+      spec.name = l7policy.name;
+      spec.description = l7policy.description;
+      spec.action = l7policy.action;
+      spec.position = l7policy.position;
+      spec.redirect_pool_id = l7policy.redirect_pool_id;
+      spec.redirect_url = l7policy.redirect_url;
+      spec.admin_state_up = l7policy.admin_state_up;
+    }
+
+    function setL7RuleSpec(l7rule) {
+      var spec = model.spec.l7rule;
+      spec.id = l7rule.id;
+      spec.type = l7rule.type;
+      spec.compare_type = l7rule.compare_type;
+      spec.key = l7rule.key;
+      spec.rule_value = l7rule.rule_value;
+      spec.invert = l7rule.invert;
+      spec.admin_state_up = l7rule.admin_state_up;
     }
 
     function setPoolSpec(pool) {
@@ -649,18 +791,9 @@
       spec.name = pool.name;
       spec.description = pool.description;
       spec.protocol = pool.protocol;
-      spec.method = pool.lb_algorithm;
+      spec.lb_algorithm = pool.lb_algorithm;
       spec.admin_state_up = pool.admin_state_up;
-      if (angular.isObject(pool.session_persistence)) {
-        var type = pool.session_persistence.type;
-        var cookie = pool.session_persistence.cookie_name;
-        if (type) {
-          spec.type = type;
-        }
-        if (type === 'APP_COOKIE' && cookie) {
-          spec.cookie = cookie;
-        }
-      }
+      spec.session_persistence = pool.session_persistence;
     }
 
     function setMembersSpec(membersList) {
@@ -671,12 +804,14 @@
         members.push({
           id: member.id,
           address: member.address,
-          subnet: mapSubnetObj(member.subnet_id),
-          port: member.protocol_port,
+          subnet_id: mapSubnetObj(member.subnet_id),
+          protocol_port: member.protocol_port,
           weight: member.weight,
           monitor_address: member.monitor_address,
           monitor_port: member.monitor_port,
           admin_state_up: member.admin_state_up,
+          backup: member.backup,
+          name: member.name,
           allocatedMember: true
         });
       });
@@ -687,14 +822,15 @@
       var spec = model.spec.monitor;
       spec.id = monitor.id;
       spec.type = monitor.type;
-      spec.interval = monitor.delay;
+      spec.delay = monitor.delay;
       spec.timeout = monitor.timeout;
-      spec.retry = monitor.max_retries;
-      spec.retry_down = monitor.max_retries_down;
-      spec.method = monitor.http_method;
-      spec.status = monitor.expected_codes;
-      spec.path = monitor.url_path;
+      spec.max_retries = monitor.max_retries;
+      spec.max_retries_down = monitor.max_retries_down;
+      spec.http_method = monitor.http_method;
+      spec.expected_codes = monitor.expected_codes;
+      spec.url_path = monitor.url_path;
       spec.admin_state_up = monitor.admin_state_up;
+      spec.name = monitor.name;
     }
 
     function onGetHealthMonitor(response) {
@@ -742,9 +878,9 @@
 
     function initSubnet() {
       var subnet = model.subnets.filter(function filterSubnetsByLoadBalancer(s) {
-        return s.id === model.spec.loadbalancer.subnet;
+        return s.id === model.spec.loadbalancer.vip_subnet_id;
       })[0];
-      model.spec.loadbalancer.subnet = subnet;
+      model.spec.loadbalancer.vip_subnet_id = subnet;
     }
 
     function mapSubnetObj(subnetId) {
